package edu.umd.dawn.common.logging;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.nio.charset.Charset;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.Node;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.layout.AbstractStringLayout;
import org.apache.logging.log4j.message.ObjectMessage;

@Plugin(name = "DawnLayout", category = Node.CATEGORY, elementType = Layout.ELEMENT_TYPE, printObject = true)
public class DawnLayout extends AbstractStringLayout {

    Boolean pretty = false;

    protected DawnLayout(Charset charset) {
        super(charset);
    }

    protected DawnLayout(Charset charset, Boolean pretty) {
        super(charset);
        this.pretty = pretty;
    }

    @PluginFactory
    public static DawnLayout createLayout(
            @PluginAttribute("charset") Charset charset,
            @PluginAttribute(value = "pretty", defaultBoolean = false) Boolean pretty) {
        return new DawnLayout(charset, pretty);
    }

    @Override
    public String toSerializable(LogEvent event) {
        Log log;
        if (event.getLevel().toString().equals("REQUEST")) {
            log = (RequestLog) ((ObjectMessage) event.getMessage()).getParameter();
            log.fillSharedFields(event);
        } else {
            log = ServiceLog.fromLogEvent(event);
        }

        try {
            if (pretty) {
                return new ObjectMapper().writerWithDefaultPrettyPrinter().writeValueAsString(log) + "\n";
            } else {
                return new ObjectMapper().writeValueAsString(log) + "\n";
            }
        } catch (JsonProcessingException e) {
            return "";
        }
    }
}
